==> haystack-0.4.19.tar.gz: lib/src/io/HJsonWriter.dart <==
import 'HGridWriter.dart';
import '../HGrid.dart';
import '../HStr.dart';
import '../HBool.dart';
import '../HVal.dart';

/**
 * HJsonWriter is used to write grids in JavaScript Object Notation.
 * It is a plain text format commonly used for serialization of data.
 ** It is specified in RFC 4627.
 *
 * @see <a href='http://project-haystack.org/doc/Json'>Project Haystack</a>
 */
class HJsonWriter implements HGridWriter {

//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  /** Write using UTF-8 */
  HJsonWriter(StringSink this._out);

/** Write a grid to an in-memory a string */
  static String gridToString(HGrid grid) {
    StringBuffer out = new StringBuffer(/*grid.numCols * grid.numRows * 32*/);
    new HJsonWriter(out).writeGrid(grid);
    return out.toString();
  }

//////////////////////////////////////////////////////////////////////////
// HGridWriter
//////////////////////////////////////////////////////////////////////////

/** Write a grid */
  void writeGrid(HGrid grid) {
    // grid begin
    _out.write("{\n");

    // meta
    _out.write("\"meta\": {\"ver\":\"2.0\"");
    _writeDictTags(grid.meta, false);
    _out.write("},\n");

    // columns
    _out.write("\"cols\":[\n");

    for (int i=0; i<grid.numCols; ++i) {
      if (i > 0) _out.write(",\n");
      HCol col = grid.cols[i];
      _out.write("{\"name\":");
      _out.write(HStr.toCode(col.name));
      _writeDictTags(col.meta, false);
      _out.write("}");
    }
    _out.write("\n],\n");

    // rows
    _out.write("\"rows\":[\n");

    for (int i=0; i<grid.numRows; ++i) {
      if (i > 0) _out.write(",\n");
      _writeDict(grid.row(i));
    }
    _out.write("\n]\n");

    // grid end
    _out.write("}\n");
  }

  void _writeDict(HDict dict) {
    _out.write("{");
    _writeDictTags(dict, true);
    _out.write("}");
  }

  void _writeDictTags(HDict dict, bool first)
  {
    dict.forEach((name, val) {
      if (first) first = false; else _out.write(", ");
      _out.write(HStr.toCode(name));
      _out.write(":");
      _writeVal(val);
    });
  }

  void _writeVal(HVal val) {
    if (val == null) _out.write("null");
    else if (val is HBool) _out.write(val);
    else _out.write(HStr.toCode(val.toJson()));
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  StringSink _out;
}

==> haystack-0.6.1.tar.gz: lib/src/io/HJsonWriter.dart <==
import 'HGridWriter.dart';
import '../HGrid.dart';
import '../HStr.dart';
import '../HBool.dart';
import '../HVal.dart';

/**
 * HJsonWriter is used to write grids in JavaScript Object Notation.
 * It is a plain text format commonly used for serialization of data.
 ** It is specified in RFC 4627.
 *
 * @see <a href='http://project-haystack.org/doc/Json'>Project Haystack</a>
 */
class HJsonWriter implements HGridWriter {
//////////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////////

  /** Write using UTF-8 */
  HJsonWriter(StringSink this._out);

/** Write a grid to an in-memory a string */
  static String gridToString(HGrid grid) {
    StringBuffer out = new StringBuffer(/*grid.numCols * grid.numRows * 32*/);
    new HJsonWriter(out).writeGrid(grid);
    return out.toString();
  }

//////////////////////////////////////////////////////////////////////////
// HGridWriter
//////////////////////////////////////////////////////////////////////////

/** Write a grid */
  void writeGrid(HGrid grid) {
    // grid begin
    _out.write("{\n");

    // meta
    _out.write("\"meta\": {\"ver\":\"2.0\"");
    _writeDictTags(grid.meta, false);
    _out.write("},\n");

    // columns
    _out.write("\"cols\":[\n");

    for (int i = 0; i < grid.numCols; ++i) {
      if (i > 0) _out.write(",\n");
      HCol col = grid.cols[i];
      _out.write("{\"name\":");
      _out.write(HStr.toCode(col.name));
      _writeDictTags(col.meta, false);
      _out.write("}");
    }
    _out.write("\n],\n");

    // rows
    _out.write("\"rows\":[\n");

    for (int i = 0; i < grid.numRows; ++i) {
      if (i > 0) _out.write(",\n");
      _writeDict(grid.row(i));
    }
    _out.write("\n]\n");

    // grid end
    _out.write("}\n");
  }

  void _writeDict(HDict dict) {
    _out.write("{");
    _writeDictTags(dict, true);
    _out.write("}");
  }

  void _writeDictTags(HDict dict, bool first) {
    dict.forEach((name, val) {
      if (first)
        first = false;
      else
        _out.write(", ");
      _out.write(HStr.toCode(name));
      _out.write(":");
      _writeVal(val);
    });
  }

  void _writeVal(HVal val) {
    if (val == null)
      _out.write("null");
    else if (val is HBool)
      _out.write(val);
    else
      _out.write(HStr.toCode(val.toJson()));
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  StringSink _out;
}
